(* -----------------------------------------------------------------------------
  RotateAndCrop        : 1.0
  Copyright  2016     : Copyright Adirondack Software & Graphics
  Last Modification    : 05-20-2016
  Source File          : uRotateAndCrop.pas
  Compiler             : Delphi10Seattle
  PlatForm             : VCL Windows 64-Bit
  Operating System     : Windows 10
  This file is copyright  W W Miller, 1986-2016
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  --------------------------------------------------------------------------- *)
unit uRotateAndCrop;

interface

uses
  Windows, Forms, Graphics, StdCtrls, ComCtrls, Classes, Controls, imageenview,
  hyiedefs, hyieutils, math, iexBitmaps,
  iesettings, Dialogs, ExtDlgs, Menus, ExtCtrls, ieview, SysUtils,
  ieopensavedlg, imageenproc, Contnrs;

type
  TFormRotateAndCrop = class(TForm)
    Timer1: TTimer;
    GroupBox1: TGroupBox;
    lblRotation: TLabel;
    trkRotate: TTrackBar;
    lblDegrees: TLabel;
    btnCrop: TButton;
    grpOptions: TGroupBox;
    lblSource: TLabel;
    cmbCropAlgorithm: TComboBox;
    chkGuidelines: TCheckBox;
    GroupBox2: TGroupBox;
    ImageEnView1: TImageEnView;
    Ok1: TButton;
    Cancel1: TButton;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure trkRotateChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure btnCropClick(Sender: TObject);
    procedure chkGuidelinesClick(Sender: TObject);
    procedure cmbCropAlgorithmChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    function GetSelectedCropAlgorithm: TIECropAlgorithm;
    procedure SetSelectedCropAlgorithm(const Value: TIECropAlgorithm);
  public
    fSourceBitmap: TIEBitmap;
    procedure UpdateView();
    property SelectedCropAlgorithm: TIECropAlgorithm
      read GetSelectedCropAlgorithm write SetSelectedCropAlgorithm;
  end;

var
  FormRotateAndCrop: TFormRotateAndCrop;

implementation

uses
  iexHelperFunctions;

{$R *.dfm}

procedure TFormRotateAndCrop.btnCropClick(Sender: TObject);
begin
  ShowTempHourglass;
  ImageEnView1.LockPaint;
  try
    ImageEnView1.IEBitmap.Assign(fSourceBitmap);
    ImageEnView1.Proc.RotateAndCrop(trkRotate.Position, ierBicubic,
      fSourceBitmap.Height / fSourceBitmap.Width, SelectedCropAlgorithm);
    fSourceBitmap.Assign(ImageEnView1.IEBitmap);
    trkRotate.Position := 0;
  finally
    ImageEnView1.UnLockPaint;
  end;
end;

procedure TFormRotateAndCrop.FormDestroy(Sender: TObject);
begin
  FreeAndNil(fSourceBitmap);
end;

procedure TFormRotateAndCrop.FormShow(Sender: TObject);
begin
  fSourceBitmap.Assign(ImageEnView1.IEBitmap);
end;

procedure TFormRotateAndCrop.FormCreate(Sender: TObject);
begin
  fSourceBitmap := TIEBitmap.Create;
  SelectedCropAlgorithm := iecaAngledPhoto;
end;

procedure TFormRotateAndCrop.chkGuidelinesClick(Sender: TObject);
begin
  IEGlobalSettings().GuidelineCount := 4;
  IEGlobalSettings().GridPen.Color := clSilver;
  IEGlobalSettings().GridPen.Style := psDot;
  IEGlobalSettings().GridPen.Mode := pmCopy;

  if chkGuidelines.checked then
    ImageEnView1.DisplayGridKind := iedgGuideLines
  else
    ImageEnView1.DisplayGridKind := iedgNone;
end;

procedure TFormRotateAndCrop.cmbCropAlgorithmChange(Sender: TObject);
begin
  UpdateView();
end;

function TFormRotateAndCrop.GetSelectedCropAlgorithm: TIECropAlgorithm;
begin
  Result := TIECropAlgorithm(cmbCropAlgorithm.ItemIndex);
end;

procedure TFormRotateAndCrop.SetSelectedCropAlgorithm
  (const Value: TIECropAlgorithm);
begin
  cmbCropAlgorithm.ItemIndex := ord(Value);
end;

procedure TFormRotateAndCrop.Timer1Timer(Sender: TObject);
begin
  UpdateView;
end;

procedure TFormRotateAndCrop.trkRotateChange(Sender: TObject);
begin
  Timer1.Enabled := False;
  Timer1.Enabled := True;
end;

procedure TFormRotateAndCrop.UpdateView();
begin
  if ImageEnView1.IEBitmap.IsEmpty() then
    exit;

  Timer1.Enabled := False;
  ImageEnView1.LockUpdate;
  try
    ImageEnView1.IEBitmap.Assign(fSourceBitmap);
    lblDegrees.Caption := IntToStr(trkRotate.Position) + '';
    ImageEnView1.Proc.RotateAndCrop(trkRotate.Position, ierNone,
      fSourceBitmap.Height / fSourceBitmap.Width, SelectedCropAlgorithm);
  finally
    ImageEnView1.UnlockUpdate;
  end;
end;

end.
